package	ck_alien;

use strict;
require	Exporter;
use	Socket;
use	util::logger;
use	Digest::MD5 qw ( md5_hex );
use	lib '/home/dimmo/main';
use	jury_common;
use	j_config;

our @ISA = qw ( Exporter );
our @EXPORT = qw( launch check retrieve );
our $log;

$log = new util::logger( "$LOGDIR/ck_alien.log" ) or die "error: can't create logfile: $!\n";
$log->info( "INIT: Started: $0" );

END {
	$log->info( "END: Retval: $?" );
}

my $COST = 50;

sub launch
{
	my $tid = shift;
	$log->info("Launch called with team id $tid");
	my ( $BANK_HOST, $BANK_PORT, $MIL_PORT, $MIL_HOST ) = ( 'jury.ructf', '2020', '6332', '' );
	my ( $address, $ck_id, $tip );

	socket ( S, PF_INET, SOCK_STREAM, getprotobyname ( 'tcp' ) ) or do {
		my @r = ( 0, $! );
		$log->error("Could not create socket: $!");
		return @r;
	};
	$address = sockaddr_in ( $BANK_PORT, inet_aton ( $BANK_HOST )  );
	$log->info("Connecting to Bank ...");
	connect ( S, $address ) or do {
		my @r = ( 4, "can't connect to bank: $!" );
		$log->error("Could not connect to Bank");
		return @r
	};
	$log->info("	Done");
	select S;
	$| = 1;
	$log->info("Sending Bank \"get_check $COST\"");
	print S "get_check $COST\r\n";
	$ck_id = <S>;
	unless ( $ck_id =~ /^\+/ )
	{
		$log->error("Wrong answer from Bank");
		return ( 0, 'Wrong answer from Bank' );
	}
	$ck_id =~ s/^\+(\S*)/$1/;
	close S;
	$log->info("Got check number: $ck_id");
	select STDOUT;
	
	socket ( S, PF_INET, SOCK_STREAM, getprotobyname ( 'tcp' ) ) or do {
		my @r = ( 0, $! );
		$log->error("could not create socket: $!");
		return @r;
	};
	$log->info("Request Mil address");
	( $tip, $MIL_HOST ) = getTeamIPs ( $tid );
	$log->info("Got Mil address : $MIL_HOST");
	$address = sockaddr_in ( $MIL_PORT, inet_aton ( $MIL_HOST )  );
	$log->info("Connecting to Mil at $MIL_HOST ...");
	connect ( S, $address ) or do {
		my @r = ( 4, $! );
		$log->error("Could not connect to Mil");
		return @r;
	};
	$log->info("	Done");
	select S;
	$| = 1;
	print S "$ck_id\r\n";
	$log->info("Sending Mil \"$ck_id\" ...");
	my ( $result, $satid, $authstr, $secretid );
	( $result, $satid, $authstr, $secretid ) = <S>;
	s/[\r\n]+$// for ( $result, $satid, $authstr, $secretid );
	$log->info("Got result : $result ; satid : $satid ; authstr : $authstr ; secretid : $secretid");
	if ( $result )
	{
		$log->warning( "Mil returned '$result'");
		return ( 3, "Mil said: '$result'" );
	}
	$secretid =~ s/.+(.{32})/$1/;
	$log->info("Real secretid : $secretid");
	close S;
	select STDOUT;
	$log->info("All done, no errors");
	( '1', "$satid;$authstr;$secretid" );
}

sub check
{
	$log->info("Check called");
	my $teamID = shift;
	my ( $tip, $tipalien ) = getTeamIPs ( $teamID );
	my $CHECK_PORT = '6332';

	my $_t = time;
	`nc $tipalien 65535 -w5 </dev/null &>/dev/null`;
	return ( 4, 'flag service is down' ) if time - $_t > 4 or $?;

	$_t = time;
	`echo -e '\r\n\r\n' | nc $tipalien 935 -w5 </dev/null &>/dev/null`;
	return ( 4, 'web service is down' ) if time - $_t > 3 or $?;

	socket ( S, PF_INET, SOCK_STREAM, getprotobyname ( 'tcp' ) ) or do
	{
		my @r = ( 0, $! );
		$log->error("Could not create socket: $!");
		return @r
	};
	my $address = sockaddr_in ( $CHECK_PORT, inet_aton ( $tipalien )  );
	$log->info("Connecting to Mil at $tipalien ...");
	connect ( S, $address ) or do
	{
		my @r = ( 4, $! );
		$log->error("Could not connect to Mil");
		return @r;
	};
	$log->info("	Done");
	select S;
	$| = 1;	
	my $r;
	$r .= chr 16 * rand for 1..200;
	$r =~ tr/\x00-\x0f/0-9A-F/;
	print S "$r\r\n";
	$log->info("Sending Mil $r ...");
	
	my $a = <S>;
	$a =~ s/[\r\n]+$//;

	$log->info("Got answer : $a");
	my ( $s, @a );
	select STDOUT;
	close S;
	for (0..3)
	{
		my $b = pack 'H200', $r;
		my @b = unpack 'C200', $b;
		$b[$_] = ~$b[$_];
		$b = pack 'C200', @b;
		my $r1 = unpack 'H200', $b;
		$r1 =~ tr/abcdef/ABCDEF/;
		$s = md5_hex $r1;
		$s =~ tr/abcdef/ABCDEF/;
		
		my $code = ( $_ < 0 || $_ > 0 ) ? 3 : $_+1;
		if ( $s eq $a )	{
			return ( $code, "Mil return $_")
		}

	}

	return ( 3, "Mil returned '$_'" );
}

sub retrieve
{
	$log->info("Retrieve called");
	my $MIL_PORT = '6332';
	my ( $teamID, $flagID, $flag ) = @_;

	$log->info("Request Mil address");
	my ( $tip, $MIL_HOST ) = getTeamIPs ( $teamID );
	$log->info("Got Mil address : $MIL_HOST");

	socket ( S, PF_INET, SOCK_STREAM, getprotobyname ( 'tcp' ) ) or do
	{
		my @r = ( 0, $! );
		$log->error("Could not create socket: $!");
		return @r;
	};
	my $address = sockaddr_in ( $MIL_PORT, inet_aton ( $MIL_HOST )  );
	$log->info("Connecting to Mil at $MIL_HOST ...");

	connect ( S, $address ) or do
	{ 
		my @r = ( 4, $! );
		$log -> error("Could not connect to Mil");
		return @r;
	};

	$log->info("	Done");
	select S;
	$| = 1;
	my $r = ( split /[:;]/, $flag ) [ 0 ];
	$log->info("Sending Mil $r");
	print S "$r\r\n";

	my $a = <S>;
	$a =~ s/[\r\n]+$//;

	close S;
	select STDOUT;
	$a =~ tr/ABCDEF/abcdef/;
	$log->info("Got ansver $a");
	$log->info(md5_hex ( ( split /[:;]/, $flag ) [ 1 ] ) );
	$log->info( ( split /:/, $flag ) [ 1 ] );
	if ( $a eq md5_hex ( ( split /[:;]/, $flag ) [ 1 ] ) ) { $log->info("returned code 1"); return 1; }
	elsif ( $a eq md5_hex "0" ) { $log->info("returned code 3"); return ( 3, "Could not open file with flags" ); }
	elsif ( $a eq md5_hex "1" ) { $log->info("returned code 2"); return ( 2, "Flag not found" ); }
	else {
		$log->info( "returned code 3" );
		return ( 3, "Unknown Mil answer: '$a'" );
	}
}

1;

