#!/usr/bin/perl
package ck_data;
use Socket;
use lib '/home/dimmo/main';
use jury_common;
use j_config;
use util::logger;
use Exporter;
use bigint;

our @EXPORT = qw(check newRound store retrieve);
our @ISA = qw(Exporter);

#service constants {START}
my $serviceLog = "$LOGDIR/ck_data.log";
my $serviceName = "[ck_data]";
my $port = 30000;
my $bible = "b64bible.txt";
my $bigPrime = 1000000007;
my $base = 5;
#service constants {END}

#service variables {START}
my %srvRoundFlag = ();
my %installedFlag = ();
my @flags = ();
my $roundID = undef;
#service variables {END}

#service dcalrations {START}
my $logger;
$logger = new util::logger($serviceLog);
$logger -> info( "$serviceName INIT: pid = $0" );
&loadBible();
#service declarations {END}

END {
  $logger -> info("$serviceName END: retval = $?; lasterror = $!");
}

sub BUFSIZE() {1024}
sub MAXINT()  {2123456}
sub sendMsg
{
  my ($tID, $msg) = @_;
  my ($ip, $answer);
  ($ip) = jury_common::getTeamIPs($tID);
  $logger -> info("$serviceName SEND MESSAGE: jury => team $tID (IP:$ip)\t\tMESSAGE:\n$msg");
  socket SOCK, PF_INET, SOCK_STREAM, getprotobyname 'tcp' or return (0, $!);
  connect SOCK, sockaddr_in($port, inet_aton($ip)) or return (4, $!);
  send SOCK, $msg, 0;
  recv SOCK, $answer, BUFSIZE, 0;
  $logger -> info("$serviceName RECV MESSAGE: team $tID => jury\t\tMESSAGE:\n$answer");
  return (1, $answer);
}

#$a, $power, $n --> $a ** $power % $n
sub getPower
{
  my ($a, $power, $n) = @_;
  my $res = 1;
  while ($power > 0)
  {
    $res = ($res * $a) % $n if ($power & 1);
    $a = ($a * $a) % $n;
    $power >>= 1;
  }
  return $res;
}
sub loadBible
{
  open BIBLE, "<$bible";
  for (<BIBLE>)
  {
    chomp $_;
    push @flags, $_;
  }
  close BIBLE;
}
sub SpId2Int
{
  my ($sputnikID) = @_;
  return (substr $sputnikID, 1) + ord(substr $sputnikID, 0, 1);
}

#$roundId, $sputnikId, $initValue --> $flagValue
sub getFlag
{
  my ($roundID, $sputnikID, $initValue) = @_;
  my ($phIdx);
  $sputnikID = SpId2Int($sputnikID);
  $phIdx = getPower($base, $roundID + $sputnikID + $initValue, $bigPrime);
  $logger -> info( "phIdx($roundID, $sputnikID, $initValue): $phIdx, " );
  $phIdx %= (scalar @flags);
  $logger -> info( "$phIdx$/" );
  $logger -> info("$serviceName FLAG IDX: $phIdx");
  return $flags[$phIdx];
}

#$tid, $time
sub newRound
{
  my ($tID, $time) = @_;
  my ($msg, $code, $answer);
  $srvRoundFlag{$tID} = {};
  $roundID = $time;
  jury_common::setVariable($serviceName, "roundID_$tID", $roundID);
  $logger -> info("$serviceName NEW ROUND team: $tID; time: $time");
  $msg = "setround $time";
  $msg = `echo $msg | gpg --clearsign`;
  ($code, $answer) = sendMsg $tID, "$msg\n";
  return ($code, $answer) unless ($code == 1);
  return (2, "non ok server answer") unless ($answer =~ /\+ok/);
  return (1, $answer);
}
#$tid
sub check
{
  my ($teamID) = @_;
  my ($msg);
  $logger -> info("$serviceName CHECK team: $teamID");
  $msg = "getdata 0 0\n";
  ($code, $answer) = sendMsg $teamID, $msg;
  return ($code, $answer);
}

#$tid, $fid, $flag
sub store
{               
  my ($teamID, $flagID, $flag) = @_;
  my ($sputnikID, $sputnikAuth, $initValue, $flag, $code, $answer, $roundID);
  $logger -> info("$serviceName STORE: team: $teamID; flagID: $flagID; flag: $flag");
  $sputnikID = $flagID;
  $roundID = jury_common::getVariable($serviceName, "roundID_$teamID");
  $sputnikAuth = jury_common::getSputnikAuth($sputnikID);
  return (0,"undefined getSputnikAuth($sputnikID)") unless $sputnikAuth;
  $initValue = 1;
  while(1)
  {
    $logger -> info("\t\tprobe INITVALUE: $initValue");
    $flag = getFlag $roundID, $sputnikID, $initValue;
    my $retval = jury_common::getFlagInfo($flag);
    $logger -> info ("getFlagInfo($flag) return: $retval");
    next if $retval;
    $logger -> info("$serviceName \t\tSAVE FLAG: $flag");
    ($code, $answer) = sendMsg $teamID, "setdata $sputnikID $sputnikAuth $initValue\n";
    return ($code, $answer) unless ($code == 1);
    next if ($answer =~ /-warning: try again/);
    last;
  } continue { $initValue = int rand MAXINT}
  return (2, "incorrect server answer") unless ($answer =~ /\+ok data committed/);
  return (1, $flag);
}

#$tid, $fid, $flag
sub retrieve
{
  my ($teamID, $flagID, $flag) = @_;
  my ($sputnikID, $answer, $code, $sputnikAuth);
  $logger -> info("$serviceName RETRIEVE: team: $teamID; flagID: $flagID; flag: $flag");
  #$sputnikID = $srvRoundFlag{$teamID}->{$flagID};
  $sputnikID = $flagID;
  return (0, "can't find sputnikID with flagID = $flagID") unless ($sputnikID);
  $sputnikAuth = jury_common::getSputnikAuth($sputnikID);
  return (0, "can't get sputnikAUTH [sputnikID = $sputnikID") unless ($sputnikAuth);
  ($code, $answer) = sendMsg $teamID, "getdata $sputnikID $sputnikAuth\n";
  return ($code, $answer) unless ($code == 1);  
  $answer =~ /\+ok data: (.+)/;
  return (2, "incorrect server answer") unless ($answer =~ /\+ok data: (.+)/ and $1 eq $flag);
  return (1, $answer);
}

1;
