package ck_start;

use strict;
use Socket;
use jury_common;
use LWP::UserAgent;
use Exporter;
use util::logger;

my $client	= LWP::UserAgent->new;
my $SAT_COST	= 25;
my $BANK_IP	= 'jury.ructf';
my $BANK_PORT	= 2020;

our @ISA	= qw( Exporter );
our @EXPORT	= qw(
	newRound
	launch
	check
);

sub newRound
{
	my $tid = $_[0];
	my @ips = getTeamIPs( $tid );
	
	@ips or return( 0, 'getTeamIPs( ) failed' );

	my $tip = $ips[0];
	my $time = $_[1];
	
        my $response = $client->get("http://$tip:7112/?Time=$time");

	return ( 1, 'ok' ) if $response->is_success && $response->content =~ /<result>0<\/result>/;
	
	return ( 4, $response->status_line.': '.$response->content ) if ! $response->is_success;

	my $msg = $response->content;

	return ( 3, 'Bad response. '.parseResp( $response->content ) );
}

sub launch
{
	my $tid = $_[0];
	my @ips = getTeamIPs($tid);

	@ips or return( 0, 'getTeamIPs( ) failed' );
	
	my $tip = $ips[0];

#	warn "  >> launch( ) called\n";

	socket(hBank, PF_INET, SOCK_STREAM, getprotobyname('tcp'));
	my $bank_addr = inet_aton($BANK_IP);
	my $sin = sockaddr_in($BANK_PORT, $bank_addr);
	connect(hBank, $sin) or return ( 0, "bank failure: connect: $!" );

	send(hBank, "get_check $SAT_COST\r\n", 0);
	recv(hBank, my $resp, 1000, 0);
	close(hBank);

#	warn "  >>   bank has given check\n";

	$resp =~ s/[\r\n]+$//g;

	$resp=~/^\+(\S+)/ or return ( 0, 'bank fail. response: '.$resp );

	my $check_id = $1;

	my $response = $client->get( "http://$tip:7112/?CheckID=$check_id" );

#	warn "  >>   get ok: ".$response->content;

	if ( $response->is_success && $response->content =~ /<result>0<\/result>/ )
	{
		$response->content =~ /<SatID>(.+)<\/SatID>.*<authstr>(.+)<\/authstr>/si;
		if ( substr($1,0,1) eq $tid ) {
			return ( 1, "$1;$2");
		}
		else {
			return( 3, "Wrong 1'st symbol of SputnikID: '$1'" );
		}
	}

	$response->is_success or return( 4, $response->status_line );

	return ( 3, 'Bad response. '.parseResp( $response->content ) );
}

sub check
{
	my $tid = $_[0];
	my @ips = getTeamIPs($tid);
	@ips or return( 0, 'getTeamIPs( ) failed' );

	my $tip = $ips[0];

	my $response = $client->get("http://$tip:7112/?ping=8");
	
	return ( 1, 'ok' ) if $response->is_success && $response->content eq 'echo';

	return ( 4, $response->status_line ) if ! $response->is_success;

	return ( 3, 'Bad response. '.parseResp( $response->content ) );
}

sub parseResp
{
	my $a = pop;
 	$a =~ m|<result>(.+?)</result>|;
	if ( defined $1 ) {
		my $tmp = $1;
		$a =~ m|<comment>(.+?)</comment>|;
		$a = "$tmp. $1";
	}
	$a;
}

