#!/bin/bash

# ------------ some config ---------------

OUT_DIR='../backup'
SRC_DIRS='.'

SCP_HOST='ftp.ructf'
SCP_USER='dimmo'
SCP_DIR='backup'

. db/config.sh

# ----------------------------------------

echo "$DB_HOST:$DB_PORT:"'*'":$DB_USER:$DB_PASS" > $PGPASS
chmod go-rwx $PGPASS

if [ -z $1 ]; then
	echo 'usage: cksys.bak.sh <mode> [restore_name]'
	echo 'mode:'
	echo '    b - backup'
	echo '    r - restore'
	exit 1
fi

if [ ! -d $OUT_DIR ]; then
	echo "create dir: $OUT_DIR"
	mkdir $OUT_DIR
fi

if [ $1 == 'b' ]; then

	# 1. Backup files

	f_name=$OUT_DIR/`date "+%F_%T"`.files.tgz

	echo -n "  create: $f_name ... "
	tar -czf $f_name $SRC_DIRS
	echo done

	# 2. Backup DB

	db_name=$OUT_DIR/`date +%F_%T`.db.gz

	echo -n "  create: $db_name ..."
	pg_dump -d $DB_NAME -h $DB_HOST -U $DB_USER | gzip > $db_name
	echo done

	# 3. Upload to remote host

	scp $f_name $SCP_USER@$SCP_HOST:$SCP_DIR
	scp $db_name $SCP_USER@$SCP_HOST:$SCP_DIR
fi

#if [ $1 == 'r' ]; then
#	
#	f_name=$2
#	echo -n "restore from: $2 ..."
#	gzip -cd $f_name | tar -xf -
#	echo "ok"
#	
#	db_name=$3
#	echo -n "restore db from: $3 ... "
#	gzip -cd $db_name > restore_db.sql
#	echo "ok"
#fi

rm -f $PGPASS

