-- created:  2008.04.13
-- modified: 2008.04.15 
-- modified: 2008.04.21
-- modified: 2008.04.23

create table game_time (
	game_time	integer		NOT NULL	-- seconds from '1970-01-01 00:00:00'
);

insert into game_time values ( 2025759600 );

create table teams (
	tid		char		PRIMARY KEY,
	tname		text		UNIQUE,
	tmail		text		UNIQUE,
	tip		inet		UNIQUE,
	tipalien		inet		UNIQUE,
	tnet		cidr		NOT NULL,
	tmoney		integer		NOT NULL,
	troom		text,
	active		boolean		NOT NULL
);

create unique index	idx_teams_tnet		on teams( tnet );
create unique index	idx_teams_tid		on teams( tid );

alter table 	teams	add constraint team_tmoney_positive check( tmoney >= 0 );

-------------------------------------------------------------------------------

create table sputnik (
	tid		char		REFERENCES teams( tid ),
	sid		text		NOT NULL,
	stime		integer		NOT NULL,
	sorbit		text		NOT NULL,
	pos		text		NOT NULL
);

create unique index	idx_sputnik_sid		on sputnik( sid );
create index		idx_sputnik_stime	on sputnik( stime );

-------------------------------------------------------------------------------

create table sauth (
	sid		text		PRIMARY KEY,
	sauth		text		NOT NULL,
	stime		int		NOT NULL,
	stime_real	timestamp	NOT NULL,
	flagtry		boolean		DEFAULT false
);

create index	idx_sauth_sauth		on sauth( sauth );

-------------------------------------------------------------------------------

create table reserved (
	orbit		text		NOT NULL,
	tid		char		REFERENCES teams( tid ),
	rtime		integer		NOT NULL,
	rtime_real	timestamp	NOT NULL
);

create unique index	idx_reserved_orbit	on reserved( orbit );
create index		idx_reserved_rtime	on reserved( rtime );

-------------------------------------------------------------------------------

create table flags (
	tid		char		REFERENCES teams( tid ),
	fid		text		NOT NULL,
	servid		text		NOT NULL,
	flag		text		NOT NULL,
	ftime		integer		NOT NULL,
	ftime_real	timestamp	NOT NULL	
);

alter table flags	add constraint tid_fid_servid_pkey PRIMARY KEY ( tid, fid, servid );

create unique index	idx_flags_flag		on flags( flag );
create index		idx_flags_ftime		on flags( ftime );

-------------------------------------------------------------------------------

create table flags_posted (
	tvictim		char		REFERENCES teams( tid ),
	tattacker	char		REFERENCES teams( tid ),
	fid		text		NOT NULL,
	servid		text		NOT NULL,
	flag		text		REFERENCES flags( flag ),
	ftime		integer		NOT NULL,
	ftime_real	timestamp	NOT NULL
);

create index		idx_flags_posted_flag	on flags_posted( flag );
alter table flags_posted add constraint uniq_posted_tattacker_flag UNIQUE ( tattacker, flag );

-------------------------------------------------------------------------------

create table status ( 
	tid		char		REFERENCES teams( tid ),
	servid		text		NOT NULL,
	status		integer		CHECK ( 0 <= status and status <= 4 ),
	scomment	text,
	updtime		integer		NOT NULL,
	updtime_real	timestamp	NOT NULL
);

alter table status add constraint sround_tid_servid_updtime_unique UNIQUE ( tid, servid, updtime );

create index 		idx_status_updtime	on status( updtime );

-------------------------------------------------------------------------------

create table advisories (
	tid		char		REFERENCES teams( tid ),
	advisory	text		NOT NULL,
	jury_comment	text,		-- may be NULL - if not reviewed yet!
	score		integer,	-- may be NULL - if not reviewed yet!
	time_real	timestamp	NOT NULL,
	is_public	boolean		DEFAULT false NOT NULL,
	is_reviewed	boolean		DEFAULT false NOT NULL
);

-------------------------------------------------------------------------------

create table checks (
	tid		char		REFERENCES teams( tid ),
	check_id_line	varchar(128)	NOT NULL,
	sum		integer		NOT NULL
);

create unique index 	idx_checks_check_id_line on checks( check_id_line );

-------------------------------------------------------------------------------


create table variables (
	vname		text		NOT NULL,
	servid		text		NOT NULL,
	vvalue		text,

	PRIMARY KEY ( vname, servid )
);

--------------------------------------------------------------

create table svcs ( 
	svc		text		NOT NULL
);

insert into svcs values ( 'alien' );
insert into svcs values ( 'control' );
insert into svcs values ( 'data' );
insert into svcs values ( 'gagarin' );
insert into svcs values ( 'start' );
insert into svcs values ( 'web' );
insert into svcs values ( 'spy' );

CREATE VIEW postedflagscount AS 
SELECT tname,count(tattacker) AS flagcnt
FROM flags_posted right JOIN teams ON tid = tattacker
WHERE active
GROUP BY tname;


CREATE VIEW advsum AS
SELECT tname,sum(coalesce(score,0)) AS score,count(advisories) as count
FROM advisories RIGHT JOIN teams ON teams.tid=advisories.tid
WHERE active
GROUP BY tname;

CREATE VIEW ss AS
SELECT tname, svc, 
 count(CASE WHEN status =1 THEN 1 ELSE null END) AS up,
 count(CASE WHEN status =2 THEN 1 ELSE null END) AS cur,
 count(CASE WHEN status =3 THEN 1 ELSE null END) AS scur,
 count(CASE WHEN status =4 THEN 1 ELSE null END) AS dwn 
FROM teams cross join svcs left join status on teams.tid=status.tid and servid=svc
WHERE active
GROUP BY tname,svc;

CREATE VIEW ss2 AS SELECT tname,svc,up||'/'||(cur+scur)||'/'||dwn AS stat,up AS scor FROM ss;

CREATE VIEW ss3 AS SELECT tname,sum(up*100+cur*50+scur*20)AS score FROM ss GROUP BY tname;

CREATE VIEW who AS 
SELECT a.tname AS attacker,v.tname AS victim,count(flag)
FROM teams AS a CROSS JOIN teams AS v LEFT JOIN flags_posted ON
tattacker=a.tid AND tvictim=v.tid
WHERE a.active AND v.active
GROUP BY attacker,victim;

CREATE VIEW what AS
SELECT tname,svc,COUNT(servid)
FROM teams CROSS JOIN svcs
LEFT JOIN flags_posted 
ON tattacker=tid AND servid=svc 
WHERE active
GROUP BY tname,svc;

