package jury_common;

#
# jury_common v0.0.3 - common DB routines
#
# Copyright (c) Simonov Dmitry, 2008
#

use strict;
use lib '.';
use db_config;
use DBI;
use Exporter;
use util::logger;
use j_config;

our @ISA = qw( Exporter );
our @EXPORT = qw(
	dbConnect

	getTeamID
	getTeamName
	getTeamMail
	getTeamIPs
	getSputnikInfo
	getSputniks
	getSputniksEx
	isOrbitReserved
	reserveOrbit

	getPostedFlags

	setSputnikAuth
	getSputnikAuth
	
	setSputnikPosOrbit

	getGameTime
	setGameTime

	getVariable
	setVariable

	getActiveTeams
	getFlagInfo
	addFlagPosted
	isFlagPosted
	addNewFlag
	setServiceStatus
	getSputniksToStore
	getActiveFlags

	cleanReservedOrbits
	cleanOldSputniks
	cleanOldSauths

        getAdvisoriesList
        getPostedFlagsCount
        getStatusList
        getCurStatusList
        getScoreList
	getAdvisoryStat
	getWhatStat
	getWhoStat
);

# ---------- config --------

my $logfile     = 'sql.log';

# --------------------------

my $dbh;
my $log;

sub dbConnect
{
	my $tid = shift;
	$log = new util::logger( sprintf $LOG_TEAMS_SQL, $tid ) or die "can't create team sql logger\n" if defined $tid;

	$dbh = DBI->connect
	(
		"dbi:Pg:dbname=$dbname;host=$dbhost;port=$dbport;", $dbuser, $dbpass,
		{
			AutoCommit	=> 0,
			RaiseError	=> 1,
			PrintError	=> 0,
			PrintWarn	=> 1,
		}
	) or die "error: can't connect to database\n";

	$log and $log -> info( "" );
	$log and $log -> info( "   *** CONNECTED TO DATABASE. db: '$dbname', host: '$dbhost', port: '$dbport'" );
}

# ---------------------------- Checkers API functions ----------------------------------------

sub getTeamID($) {
	_sql_one_row( 	'SELECT tid FROM teams WHERE ? << tnet', @_ );
}

sub getTeamName($) {
	_sql_one_row(	'SELECT tname FROM teams WHERE tid = ?', @_ );
}

sub getTeamMail($) {
	_sql_one_row(	'SELECT tmail FROM teams WHERE tid = ?', @_ );
}

sub getTeamIPs($) {
	_sql_one_row(	'SELECT tip, tipalien FROM teams WHERE tid = ?', @_ );
}

sub getSputnikInfo($) {
	_sql_one_row(	'SELECT tid, stime, sorbit, pos FROM sputnik WHERE sid = ?', @_ );
}

sub getSputniks($$)
{
	my @ret;
	
	if ( defined $_[0] ) {
		@ret = _sql_all_rows( 'SELECT sid FROM sputnik WHERE tid = ? AND stime >= ?', @_ );
	}
	else {
		@ret = _sql_all_rows( 'SELECT sid FROM sputnik WHERE stime >= ?', $_[1] );
	}
	
	map { $_->[0] } @ret;
}

sub getPostedFlags($)
{
	my ( $tid ) = @_;
	my @r = _sql_all_rows( 'SELECT flag FROM flags_posted WHERE tattacker = ?', $tid );
	map { $_->[0] } @r;
}

sub getSputniksEx($$)
{
	my @ret;

	if ( defined $_[0] ) {
		@ret = _sql_all_rows( 'SELECT sid, stime, sorbit, pos FROM sputnik WHERE tid = ? AND stime >= ?', @_ );
	}
	else {
		@ret = _sql_all_rows( 'SELECT sid, stime, sorbit, pos FROM sputnik WHERE stime >= ?', $_[1] );
	}
	@ret;
}

sub isOrbitReserved($) {
	_sql_one_row( 'SELECT tid FROM reserved WHERE orbit = ?', @_ );
}

sub reserveOrbit($$$) {
	_sql_do( 'INSERT INTO reserved ( tid, orbit, rtime, rtime_real ) VALUES( ?, ?, ?, now() )', @_ );;
}

sub setSputnikAuth($$) {
	_sql_do( 'INSERT INTO sauth( sid, sauth, stime, stime_real, flagtry ) VALUES( ?, ?, ?, now(), false )', @_, getGameTime() );
}

sub getSputnikAuth($) {
	_sql_one_row( 'SELECT sauth FROM sauth WHERE sid = ?', @_ );
}

sub setSputnikPosOrbit($$$$$) {
	_sql_do( 'INSERT INTO sputnik ( tid, sid,  sorbit, pos, stime ) VALUES( ?, ?, ?, ?, ? )', @_ );
}


sub getVariable($$) {
	_sql_one_row( 'SELECT vvalue FROM variables WHERE servid = ? AND vname = ?', @_ );
}

sub setVariable($$$) {
	_sql_do( 'DELETE FROM variables WHERE servid = ? AND vname = ?', @_[0,1] );
	_sql_do( 'INSERT INTO variables ( servid, vname, vvalue ) VALUES ( ?, ?, ? )', @_ );
}

# ------------------------------ End of checkers API -----------------------------------


# --------------------------- Functions for internal use -------------------------------

sub getActiveTeams() {
	map { $_->[0] } _sql_all_rows( 'SELECT tid FROM teams WHERE active = true' );
}

sub getFlagInfo($) {
	_sql_one_row( 'SELECT tid, fid, servid, ftime FROM flags WHERE flag = ?', @_ );
}

sub isFlagPosted($$) {
	_sql_one_row( 'SELECT 1 FROM flags_posted WHERE flag = ? AND tattacker = ?', @_ );
}

sub addFlagPosted($$) {
	my ( $flag, $tattacker ) = @_;
	my ( $tvictim, $fid, $servid ) = getFlagInfo( $flag );
	my $time = getGameTime( );

	_sql_do( 'INSERT INTO flags_posted (tvictim,tattacker,fid,servid,flag,ftime,ftime_real) VALUES (?,?,?,?,?,?,now())',
		$tvictim, $tattacker, $fid, $servid, $flag, $time );
}

sub getGameTime() {
	_sql_one_row( 'SELECT game_time FROM game_time' );
}

sub setGameTime($) {
	_sql_do( 'UPDATE game_time SET game_time = ?', @_ );
}

sub addNewFlag($$$$) {
#	_sql_do( 'DELETE FROM flags WHERE tid = ? AND fid = ? AND servid = ?', @_[0..2] );
	_sql_do( 'INSERT INTO flags ( tid, fid, servid, flag, ftime, ftime_real ) VALUES ( ?,?,?,?,?,now() )', @_, getGameTime() );
}

sub setServiceStatus($$$$) {
	_sql_do( 'INSERT INTO status ( tid, servid, status, scomment, updtime, updtime_real ) VALUES (?,?,?,?,?,now())', @_, getGameTime() );
}

sub getSputniksToStore() {
	my @ret = map { $_->[0] } _sql_all_rows( 'SELECT sid FROM sauth WHERE flagtry = false' );
	_sql_do( 'UPDATE sauth SET flagtry = true' );
	@ret;
}

sub getActiveFlags($$$) {
	_sql_all_rows( 'SELECT fid, flag FROM flags WHERE tid = ? AND servid = ? AND ftime > ?', @_ );
}

sub cleanReservedOrbits() {
	_sql_do( 'DELETE FROM reserved WHERE rtime + ? < ? ', $LIFE_RESERVED, getGameTime(), );
}

sub cleanOldSputniks() {
	_sql_do( 'DELETE FROM sputnik WHERE stime + ? < ?', $LIFE_SPUTNIK, getGameTime() );
}

sub cleanOldSauths() {
	_sql_do( 'DELETE FROM sauth WHERE stime + ? < ?', $LIFE_SPUTNIK, getGameTime() );
}

#
# -------------------------------------------------------------------------------------------------------
#

sub getPostedFlagsCount() {
        _sql_all_rows( 'SELECT tname,flagcnt FROM postedflagscount' );
}
sub getAdvisoriesList() {
	_sql_all_rows( 'SELECT tname,advisory,score,time_real,jury_comment,is_public FROM teams JOIN advisories ON teams.tid=advisories.tid where active ORDER BY time_real' );
}
sub getAdvisoryStat() {
        _sql_all_rows( 'SELECT tname,score,count from advsum' );
}
sub getCurStatusList() {
        _sql_all_rows( 'select tname,m.servid,status,scomment from teams join status on teams.tid=status.tid join (select tid,servid,max(updtime) from status group by servid,tid) as m on updtime=max AND teams.tid=m.tid AND status.servid=m.servid' );

}
sub getStatusList() {
        _sql_all_rows( 'SELECT tname,svc,stat,scor FROM ss2' );
}
sub getScoreList() {
        _sql_all_rows( 'SELECT tname,tmoney FROM teams WHERE active' );
}
sub getWhoStat() {
	_sql_all_rows( 'SELECT attacker,victim,count FROM who' );
}
sub getWhatStat() {
	_sql_all_rows( 'SELECT tname,svc,count FROM what');
}

#
# ----------------------------- I N T E R N A L   S U B R O U T I N E S ---------------------------------
#

sub _sql_one_row
{
	my $q = shift;
	_log_q( $q, @_ );

	my @ret = $dbh -> selectrow_array( $q, undef, @_ );
	_log_r( @ret );

	wantarray ? @ret : $ret[0];
}

sub _sql_all_rows
{
	my $q = shift;
	_log_q( $q, @_ );

	my $ref = $dbh -> selectall_arrayref( $q, undef, @_ );
	_log_r( @$_ ) for @$ref;

	@$ref;
}

sub _sql_do
{
	my $q = shift;
	_log_q( $q, @_ );

	my $num = $dbh -> do( $q, undef, @_ );
	$dbh -> commit( );
	_log_r( "$num ROWS AFFECTED" );

	$num;
}

#
#	_log_q - log SQL query
#

sub _log_q
{
	my ( $q, @bindvals ) = @_;
	while ( @bindvals )
	{
		my $val = sprintf "'%s'", shift @bindvals;
		$q =~ s/\?/$val/;
	}
	$log and $log -> info( "< $q" );
}

#
#	_log_q - log SQL result
#

sub _log_r
{
	local $, = "', '";
	$log and $log -> info( "> '@_'" );
}

1;

