#!/usr/bin/perl

#RuCTF Bank System v.1.1.3
#Copyright (c) 2008 Nickolai Zhuravlev
#mailto: znick@ructf.org

$/="\r\n";
$\=$/;

use DBI;
use Socket;
use IO::Handle;
use Digest::MD5 qw(md5_base64);
use Digest::SHA1 qw(sha1_base64);
use POSIX qw(getpwnam getgrnam setuid setgid);

require('config.pl') or die "FATAL: No config.pl!\n";
require('str_eng.pl') or die "FATAL: No str_eng.pl!\n";
require('sql.pl') or die "FATAL: No sql.pl!\n";

open LOG,">>$logfile" or die "Can`t open log file : $logfile : $! \n";
select LOG;
$|++;
select STDOUT;

open CHECK,">>$check_logfile" or die "Can`t open check_logfile : $check_logfile : $! \n";
select CHECK;
$|++;
select STDOUT;

socket(SERVER, PF_INET, SOCK_STREAM, getprotobyname('tcp'));
setsockopt(SERVER, SOL_SOCKET, SO_REUSEADDR, 1);
$my_addr = sockaddr_in($port, INADDR_ANY);
bind(SERVER, $my_addr);
listen(SERVER, SOMAXCONN) or die "Couldn't listen on port $server_port : $!\n";
logger("Socket on port $port created");

autoflush SERVER;

my $old_uid=$<;
my $old_gid=$(;
$user=getpwnam($user);
$group=getgrnam($group);
POSIX::setgid($group) or die "Can`t change gid! : $!" ;
POSIX::setuid($user) or die "Can`t change uid! : $!";
die "Can`t change uid. Possibly no such uid!" if ($old_uid==$();
die "Can`t change gid. Possibly no such gid!" if ($old_gid==$<);

logger("NOW UID: $( " . getpwuid($() . " GID: $< " . getgrgid($<));

our @pids;
our %params;

@_=grep (/^Pg$/,DBI->available_drivers);
die ("FATAL: No DBI driver for PostgreSQL (dbi:pg)!") if ($#_<0);


our %cmd = (
    'echo' 		=> \&cmd_echo,
    'get_check' 	=> \&cmd_get_check,
    'cache' 		=> \&cmd_cache,
    'show_check'	=> \&cmd_show_check,
    'show_status'	=> \&cmd_show_status,
    );
    
sub not_imp
{
    return $str{'ise_not_imp'};
}

logger('Started :)');

while ($client=accept(CLIENT, SERVER))
{
    my $pid=fork();
    if ($pid>0)
    {
	$SIG{INT}=\&stop_parent;
	push(@pids,$pid);
	$SIG{CHLD}='IGNORE';
	close CLIENT;
    }elsif($pid==0)
    {
	$SIG{INT}=\&stop_child;
	$SIG{CHLD}='DEFAULT';
	close SERVER;
        run_server();
    }
    

}

sub detect_tid
{
    my $ret;
    $params{'client_ip'}=shift;
    logger("Detecting tid");
    eval
    {
	sql_prepare('get_tid');
	$sth->execute;
	my $i=0;
	while(my @row = $sth->fetchrow_array())	
	{
	    $i++;
	    $ret=$row[0];
	}
	if ($i!=1)
	{
	    $ret=$cmd{'ise_detect_tid'};
            logger ($ret);
            die ($ret);
	}

	$dbh->commit;
    };
    if ($@)
    {
	$ret=$str{'ise_detect_tid'} . " $@";
	$dbh->rollback;
	$ret="Can`t detect tid! :(";
	logger ($ret);
	die ($ret);
    }
    logger("Tid: $ret");
    our $strict=1;
    $strict=0 if (exists($whitelist{$tid}));
    return $ret;
}

sub get_whitelist
{
    my @ret;
    open WHITE,$whitelist_file;
    while (<WHITE>)
    {
	chomp;
	push(@ret,$_);
	push(@ret,1);
    }
    close WHITE;
    return @ret;
}    

sub run_server
{
    $_=connect_to_sql();
    if(!($_ eq '0'))
    {
	$_="-ISE: Can`t connect to sql server $dbhost:$dbport user:$dbuser : $_";
	logger(">$_");
	print CLIENT $_;
	exit 1;
    }

    my($dport,$daddr) = sockaddr_in($client);
    my $dname = gethostbyaddr($daddr,AF_INET);
    $daddr=inet_ntoa($daddr);
    logger("Accepted client $daddr $dname:$dport");
    my $ret;
    autoflush CLIENT;
    our %whitelist=get_whitelist();    
    $params{'tid'}=detect_tid($daddr); #Team id
    logger("Team id: $params{'tid'}");

    while (<CLIENT>)
    {
	chomp;
	logger("<$_");
	split / /;
	$_=shift;
	if (exists($cmd{$_}))
	{
	    $_=$cmd{$_};
	    $ret=&$_(@_);
	    logger(">$ret");
	    print CLIENT $ret;
	}else
	{
	    $ret="$str{'no_such_cmd'} $_ @_";
	    logger(">$ret");
	    print CLIENT $ret;
	}
	
    }
}

sub connect_to_sql
{
    my $ret=0;
    $dbh = DBI->connect("dbi:Pg:dbname=$dbname;host=$dbhost;port=$dbport;" .
                          "", "$dbuser", "$dbpass") or $ret="$DBI::errstr";
    $dbh->{AutoCommit} = 0; 
    $dbh->{RaiseError} = 1;
    return $ret;
}

sub cmd_echo
{
    return "+@_";
}

sub sql_prepare
{
    my $str=shift;
    if (!exists $sql{$str})
    {
	$@="No such sql id!";
	0;
    }
    $str = $sql{$str};
    $str =~ s/\!\!(\w+?)\!\!/$params{$1}/g;
    logger("SQL: $str");
    $sth = $dbh->prepare($str);
}

sub cmd_get_check
{
    my $unique=0;
    my $ret;
    $params{'sum'}=int($_[0]);
    sleep(2) if($strict);
    return "-Sum should be positive" if ($params{'sum'}<=0);
    return "-Sum should be <=50" if (($strict)&&($params{'sum'}>50));
    
    while(!$unique)
    {
	$params{'check_id_line'}=gen_check_id_line();
    	$unique=1;
	logger("Trying to get_check sum: $params{'sum'} check: $params{'check_id_line'}");
	eval
	{
	    sql_prepare('get_check_1');
	    $sth->execute;
	    sql_prepare('get_check_2');
	    $sth->execute;	    
	    $dbh->commit;
	};


	if ($@)
	{
	    if ($@ =~ /"teams_sum_zero"/)
	    {
		$ret=$str{'not_enought_money'};
	    }elsif ($@ =~ /"idx_checks_check_id_line"/)
	    {
		$unique=0;
	    }else
	    {
		$ret=$str{'ise_check_system'};
	    }
	    $dbh->rollback;
	}else
	{
	    $ret= "+$params{'check_id_line'}"; #+OK
	}
    }
    return $ret;
}


sub cmd_cache
{
    my $ret;
    $params{'check_id_line'}=shift;
    logger("Trying to cache check: $params{'check_id_line'}");
    my $stid;
    ($_,$stid)=cmd_show_check($params{'check_id_line'},1);
    return $_ if (/^\-/);
    if (/\+(\d+)/)
    {
	$params{'sum'}=$1;
    }
    else
    {
	return $str{'ise_cache_unknown'};
    }
    logger("	Sum of check is $1");
    eval
    {
        sql_prepare('cache_1');
        if (($sth->execute) == 0)
        {
	    $ret=$str{'no_such_check'};
	    $dbh->rollback;		
	}else
	{
	    sql_prepare('cache_2');
	    $sth->execute;
	    $dbh->commit;
	}
    };
    if ($@)
    {
	$dbh->rollback;
        return $str{'ise_cache'} . " $@";
    }
    logger_check("Check  $params{'sum'} $stid -> $params{'tid'}");
    return "+$params{'sum'}";

}

sub cmd_show_check
{
    $params{'check_id_line'}=shift;
    my $for_update=shift;
    my $ret=0;
    my $stid;
    logger("Trying to check check: $params{'check_id_line'}");
    eval
    {
	if($for_update)
	{
	    sql_prepare('show_check_1_for_update');
	}else
	{
    	    sql_prepare('show_check_1');
	}
        $sth->execute;
	my $i=0;
	while(my @row = $sth->fetchrow_array())
	{
	    $i++;
	    $ret="+$row[0]";
	    $stid=$row[1];
	}
        if ($i==0)
	{
	    $ret=$str{'no_such_check'};
	    $dbh->rollback;
	}
	elsif($i>1)
	{
	    $ret=$str{'ise_show_check_more'};
	    $dbh->rollback;
	}
	$dbh->commit;
    };
    if ($@)
    {
	$ret=$str{'ise_show_check'} . " $@";
	$dbh->rollback;
    }
    return ($ret,$stid) if ($for_update);
    return $ret;
}

sub cmd_show_status
{
    my $ret;
    logger("Trying to show status");
    eval
    {
	sql_prepare('show_status_1');
	$sth->execute;
	$i=0;
	while(my @row = $sth->fetchrow_array())	
	{
	    $i++;
	    $ret="+$row[0]";
	}
	if ($i!=1)
	{
	    $ret=$cmd{'ise_show_status_too_much'};
	}

	$dbh->commit;
    };
    if ($@)
    {
	$ret=$str{'ise_show_status'} . " $@";
	$dbh->rollback;
    }
    return $ret;
}

sub gen_check_id_line
{
    open RND,'/dev/urandom';
    my $t;
    for (my $i=0;$i<=20;$i++)
    {
	$t.=getc RND;
    }

    close RND;
    $t+=time;
    $t = md5_base64($t);
    $t = sha1_base64($t);
    return "+$t";
}

sub stop_child
{
    logger("INT");
    $dbh->dissconnect;
    close LOG;
    exit 0;
}

sub stop_parent
{
    close SERVER;
    logger('Stoping now!');
    logger('Sending TERM signal to process: ' . join / /,@pids);
    kill SIGINT,@pids;
    logger('Sending KILL signal to process: ' . join / /,@pids);
    kill 9,@pids;
    logger('Dissconecting from SQL Server ...');
    logger('See YA!');
    close LOG;
    exit 0;
}

sub logger
{
    my $time=localtime;
    print LOG "$$ $time " . join (/ /,@_);
}

sub logger_check
{
    my $time=localtime;
    print CHECK "$$ $time " . join (/ /,@_);
}
