#!/usr/bin/perl

#
# sv_control v0.0.1
#
# Copyright (c) Simonov Dmitry, 2008
#

use strict;
use Socket;
use lib '..';
use lib '/home/dimmo/main';
use util::logger;
use jury_common;
use j_config;

# ----------- stub for testing ----

my $comment=<<'COMMENT';
sub getSputniks($$)
{
	my ( $tid, $time ) = @_;
	return ( 'sid_xxx', 'sid_yyy', 'sid_zzz' );
}

sub getSputniksEx($$)
{
	my @a = ( 
		[ 'sid_xxx', 100, "12 23 45", "100 200 300" ],
		[ 'sid_yyy', 120, "23 45 67", "200 100 300" ],
		[ 'sid_zzz', 130, "65 43 21", "300 200 100" ],
	);
	return @a;
}

sub getGameTime()
{
	return 1000;
}
COMMENT

# ---- end of stub for testing ----



# ----------- config --------------

my $port     = 20000;
my $logfile  = "$LOGDIR/sv_control.log";
my $timeout  = 10;			# seconds for client connection
my $nl	     = "\r\n";			# Will be sent to client
++$|;

# -------- end of config ----------

my %handlers = (
	'LIST'		=> \&h_list,
	'LISTPOSITION'	=> \&h_listposition,
);

my %msg = (
	'unk_cmd'	=> 'ERROR: unknown command',
	'timeout'	=> 'ERROR: timeout',
	'int_err'	=> 'ERROR: internal DB error. Tell jury!',
	'endlist'	=> 'ENDLIST',
);

my $log;

$log = new util::logger( $logfile ) or die "error: can't create logfile: $!\n";
$log -> info( "   *** PROCESS STARTED\n" );

print $log -> info( " config: port    = $port" );
print $log -> info( " config: logfile = $logfile" );
print $log -> info( " config: timeout = $timeout" );

$SIG{__DIE__} = sub
{
	chomp( my $msg = shift );
	
	$log -> alert( "  *** Oops! PROCESS DIED. Reason: '$msg'\n" );
	for ( 0 .. 90 ) {
		my ( $pkg, $fname, $line, $sub ) = caller( $_ ) or last;
		$fname =~ s|^.*\\||;
		$fname =~ s|^.*/||;
		$log -> alert( sprintf "    (%2d)  pkg: %-10s  file: %-25s  sub: %s( )\n", $_, $pkg, "$fname:$line", $sub );
	}
	exit 1;
};

END {
	$log -> info( "   *** PROCESS FINISHED\n" );
}

$SIG{CHLD} = 'IGNORE';									# To avoid 'zombies'

socket( S, PF_INET, SOCK_STREAM, 0 )			or die "error: socket: $!\n";
setsockopt( S, SOL_SOCKET, SO_REUSEADDR, 1 );
bind( S, sockaddr_in( $port, INADDR_ANY ) )		or die "error: bind: $!\n";
listen( S, 10 )						or die "error: listen: $!\n";

print $log->info( "startup: sv_control is listening tcp port $port" );

while ( 1 )
{
	my ( $cli_port, $cli_addr ) = sockaddr_in accept( C, S );
	my $pid = fork( );
	$pid < 0					and die "error: fork: $!\n"; 	# comment this for Windows
	if ( $pid == 0 )
	{
		dbConnect( "control.$pid" );
		close S;
		$cli_addr = inet_ntoa $cli_addr;
		print $log->info( " client: connected $cli_addr:$cli_port" );
		process_client( );
		exit 0;
	}
	close C;
}


sub process_client( )
{
	local $SIG{ALRM} = sub {
		send C, $msg{timeout}, 0;
		print $log->info( "  reply: [S->C] '$msg{timeout}'" );
		
		shutdown C, 2;
		close C;
		exit 0;
	};
	alarm $timeout;						# Windows version of perl sucks

	defined ( my $cmd = <C> ) or return;
	$cmd =~ s/[\r\n]//g;
	print $log->info( "request: [S<-C] '$cmd'\n" );
	
	if ( ! exists $handlers{ $cmd } ) {
		print $log->info( "  reply: [S->C] '$msg{unk_cmd}'" );
		send C, $msg{unk_cmd}.$nl, 0;
	}
	else {
		$handlers{ $cmd } -> ( );
	}
	
	shutdown C, 2;
	close C;
}

sub h_list( )
{
	my $time = getGameTime( );
	print $log->info( "   time: $time" );

	for ( getSputniks( undef, $time - $LIFE_SPUTNIK ) )
	{
		send C, $_.$nl, 0;
		print $log->info( "  reply: [S->C] '$_'" );
	}
	send C, "ENDLIST".$nl.$nl, 0;
}

sub h_listposition ( )
{
	my $time = getGameTime( );
	print $log->info( "   time: $time" );

	my @s_ex;							# :)

	eval {
		@s_ex = getSputniksEx( undef, $time - $LIFE_SPUTNIK );
	};

	if ( $@ )
	{
		send C, $msg{'int_err'}.$nl, 0;
		die "getSputniksEx( ) failed\n";
	}
	
	for ( @s_ex )
	{
		my ( $sid, $startTime, $sorbit, $startPos ) = @$_;
		my $answ = "$sid $sorbit $startPos $startTime";
		send C, $answ.$nl, 0;
		print $log->info( "  reply: [S->C] '$answ'" );
	}

	send C, $msg{'endlist'}.$nl.$nl, 0;
	
}

