#!/usr/bin/perl
use strict;

use lib '/home/dimmo/main/';

use Socket;
use Time::Local;
use util::logger;
use jury_common;
use j_config;
use IO::Handle;

#const
my $logfile		= 'sv_flagrecv.log';
my $port		= 31337;			# 3... 13 ...37
my $sleep		= 5;
my $cli_timeout		= 10;

my $nl			= "\r\n";

my $rrr			= join ( "\r\n",
			'                              -1',
			'     /           / 2 Pi n \\ \\ ',
			'    | inf   cos | ________ | |',
			'    | ___        \\    3   /  |',
			'(-1)| \\    _________________ |',
			'    | /__     	 n+2         |',
			'    | n=1     	2            |',
			'     \\                      / '
		);

my $ok			= '+ACCEPTED';
my $noflag		= '-NO SUCH FLAG';
my $yourflag		= '-YOUR OWN FLAG';
my $doubleflag		= '-YOU ALREADY POSTED IT';
my $oldflag		= '-FLAG EXPIRED';
my $timeout		= '-TIMEOUT';
my $unkip		= '-UNKNOWN IP';
my $tryAgain		= '-TEMPORARY PROBLEMS. TRY AGAIN';
my $msg13		= '-13 FLAG NOT ALLOWED. INSTEAD ENTER VALUE OF:'.$/.$rrr;
my $ok13		= '+13 ANSWER OK. GO ON SENDING FLAGS';

my $helo		= 'flagrecv server 0.1. please, wait ...'.$nl;
my $helo1		= 'welcome, ';
my $helo2		= 'enter your flags, one per line'.$nl;

$| = 1;

#log
my $log = util::logger->new($logfile);
$log->info( "*** INIT: Started: $0" );

END {
	$log->info( "*** END: Retval: $?" );
}

$SIG{__DIE__} = sub {
	chomp( my $msg = shift );
	$log -> alert( "  *** Oops! PROCESS DIED. Reason: '$msg'\n" );
	for ( 0 .. 90 ) {
		my ( $pkg, $fname, $line, $sub ) = caller( $_ ) or last;
		$fname =~ s|^.*\\||;
		$fname =~ s|^.*/||;
		$log -> alert( sprintf "    (%2d)  pkg: %-10s  file: %-25s  sub: %s( )\n", $_, $pkg, "$fname:$line", $sub );
	}
	exit 1;
};

#subs
sub alrmh{
	print $log->warning( " client: timeout, socket closed" );
	out( $timeout );
	close C;
	exit
}
#sub t2{
#	timelocal reverse split/[ :-]/,pop
#}

sub out
{
	my $msg = pop;
	$log -> info( "  reply: [S->C] '$msg'" );
	print C $msg.$nl;
}

#pseudo subs
#sub getGameTime{
#	return "3";
#};
#sub getTeamID{
#	return (pop eq '127.0.0.1')?1:2;
#}
#sub getFlagInfo{
#	return ('123',undef,undef,'1');
#}
#sub isFlagPosted{
#	return undef;
#}
#sub addFlagPosted{
#	return 1;
#}
	
$SIG{CHLD} = 'IGNORE';					# To avoid zombie processes

#init
socket(S,PF_INET,SOCK_STREAM,getprotobyname('tcp'))	or die "no socket $!";
setsockopt( S,SOL_SOCKET,SO_REUSEADDR,1);
bind(S,sockaddr_in($port,INADDR_ANY))			or die "no binding $!";
listen(S,SOMAXCONN)					or die "cant listen $!";

print $log->info( "startup: listening tcp port $port" );
print $log->info( " config: sleep         = $sleep sec" );
print $log->info( " config: cli_timeout   = $cli_timeout sec" );

#main
while(accept(C,S)){
	if(!fork()){
		dbConnect( "flagrecv.$$" );			# In every child process !

		# session start
		autoflush C;
		
		print C $helo;
	
		my($rport,$ip) = sockaddr_in(getpeername C);
		$ip = inet_ntoa( $ip );
		print $log->info( " client: $ip:$rport" );

		my $tid = getTeamID( $ip );
		if(!defined $tid){
			out $unkip;
			close C;
			exit
		}

		sleep( $sleep );

		my $tname = getTeamName( $tid );
		print C $helo1,$tname,$nl,$helo2;

		print $log->info( " client: team $tname" );
		
		$SIG{ALRM}=\&alrmh;
		alarm $cli_timeout;

		my $cnt=0;
loo:	while(<C>){
			#chomp( my $flag = $_ );
			/(.*)\r\n/;
			my$flag=$1;
			alarm 0;
			$log->info( "request: [S<-C] '$flag'" );

			if(++$cnt==13){
				if($flag != 14){
					print $log->warning( " client: 13 BAD answer, socket closed" );
					out $msg13;
					close C;
					exit
				}
				print $log->warning( " client: 13 GOOD answer" );
				out $ok13;
				next loo
			}

			my($vtid,undef,undef,$ftime)=getFlagInfo($flag);
			if(!defined $vtid){
				out$noflag;
				next loo;
			}
			if($tid eq $vtid){
				out$yourflag;
				next loo;
			}
			if((getGameTime())-($ftime)>$LIFE_FLAG ){
				out$oldflag;
				next loo;
			}
			if(defined isFlagPosted($flag,$tid)){
				out$doubleflag;
				next loo;
			}
			eval {
				addFlagPosted($flag, $tid);
			};

			if ( $@ ) {
				print $log->warning( "     db: error (addFlagPosted failed): $@" );
				out $tryAgain;
			}
			else {
				out $ok
			}
			alarm 10;
		}
		print $log->warning( " client: connection lost" );
		close C;
		exit;
	}
	close C;
}

