#!/usr/bin/perl
use strict;
use lib '/home/dimmo/main/';
use Socket;
use util::logger;
use jury_common;
use IO::Handle;

# const
my $logfile	= 'sv_start.log';
my $port	= 10000;
my $nl		= "\r\n";
my $helo	= "sv_start server 0.1 ready";
my $unkip	= "-UNKNOWN IP";
my $bad		= "-BAD REQUEST";
my $alr		= "-ALREADY RESERVED";
my $ok		= "+OK";
my $timeout	= "-TIMEOUT";
my $again	= '-INTERNAL ERROR. TRY AGAIN';

my $cli_timeout	= 10;

# --------------- end of const -----------------

my $log;

$log = new util::logger( $logfile ) or die "error: can't create logfile: $!";
$log -> info( "   *** PROCESS STARTED" );

END {
	$log->info( "*** END: Retval: $?" );
}

$SIG{__DIE__} = sub {
	chomp( my $msg = shift );
	$log -> alert( "  *** Oops! PROCESS DIED. Reason: '$msg'\n" );
	for ( 0 .. 90 ) {
		my ( $pkg, $fname, $line, $sub ) = caller( $_ ) or last;
		$fname =~ s|^.*\\||;
		$fname =~ s|^.*/||;
		$log -> alert( sprintf "    (%2d)  pkg: %-10s  file: %-25s  sub: %s( )\n", $_, $pkg, "$fname:$line", $sub );
	}
	exit 1;
};

$SIG{CHLD} = 'IGNORE';					# To avoid zombies


#pseudo subs
#sub getTeamID{
#	if ($_[0] ne '127.0.0.1'){return undef};
#	return 'local'}
#sub isOrbitReserved{
#	if ($_[0] eq '1'){ return 1};
#	return undef;
#}
#sub	reserveOrbit{}


sub alrmh{	#ALARMSIG handler
	out( $timeout );
	print $log->warning( "timeout" );
	close C;
	exit
}

socket( S,PF_INET,SOCK_STREAM,getprotobyname('tcp') )		or die "socket: $!\n";
setsockopt S,SOL_SOCKET,SO_REUSEADDR,1;
bind( S,sockaddr_in($port,INADDR_ANY) )				or die "bind: $!\n";
listen( S, SOMAXCONN )						or die "listen: $!\n";

print $log->info( "startup: listening port $port" );
print $log->info( " config: cli_timeout  = $cli_timeout" );

while(accept(C,S))
{
	if(!fork())
	{
		dbConnect( "start.$$" );
		autoflush C;

		send C, $helo.$nl, 0;
		$SIG{ALRM}=\&alrmh;
		
		my ( $r_port, $r_ip ) = sockaddr_in(getpeername C);
		$r_ip = inet_ntoa( $r_ip );
		print $log->info( " client: $r_ip : $r_port" );
		
		alarm 10;
		chomp( $_ = <C> );
		$log->info( "request: [S<-C] '$_'" );

		alarm 0;
		check:{			
			my $tid = getTeamID( $r_ip );
			if( !defined $tid ) {
				out( $unkip );
				last check;		#bad ip
			}
			if(!/^RESERVEORBIT \((\S+ \S+ \S+)\)/){
				out( $bad );
				last check;		#bad request
			}
			if( isOrbitReserved($1) ){
				out( $alr );
				last check;		#bad orbit
			}
			eval {
				my $time = getGameTime( );
				$log -> info( "   time: $time (game)" );
				$log -> info( "    sub: reserveOrbit( '$tid', '$1', '$time' ) " );
				reserveOrbit( $tid, $1, $time );
			}; # no fuck :)

			if ( $@ ) {
				out( $again );
				print $log->warning( "warning: reserveOrbit() failed: $@" );
				last check;
			}
			out( $ok );
		}
		print $log -> info( " client: disconnected" );
		close C;
		exit;
	}
	close C;
}

sub out {
	my $msg = shift;
	$log -> info( "  reply: [S->C] '$msg'" );
	send C, $msg.$nl, 0;
}

