#!/usr/bin/perl

#
#	ipgiver module v0.0.1 for RuCTF checksystem
#
#	Simonov Dmitry, 2008
#

#
# Do not forget to add to '/etc/sudoers'
#
#	dimmo		ALL=NOPASSWD: /sbin/ifconfig
#	dimmo		ALL=NOPASSWD: /sbin/ip
#

package util::ipgiver;

use strict;
use lib '/homme/dimmo/main';
use util::logger;
use j_config;
use Exporter;
use Socket;

our @ISA = qw( Exporter );
our @EXPORT = qw(
	$IPGIVER_ENABLED
	ipGetIfaces
	ipSetAlias
	ipSetRandAlias
	ipUnsetAlias
);

# ------------- config -----------------

our $IPGIVER_LOG	= "$LOGDIR/ipgiver.log";
our $IPGIVER_ENABLED	= 0;				# 1 - enabled, 0 - disabled
our $IPGIVER_HOST	= '10.254.0.254';		# i'll ask him for ip
our $IPGIVER_PORT	= 12345;
our $IPGIVER_ALIAS	= 'eth0:1';
our $IPGIVER_TIMEOUT	= 5;

# ----------- end of config ------------

my $log = new util::logger( $IPGIVER_LOG )			or die "error: util::logger( $IPGIVER_LOG ) failed\n";
$log -> info( '-'x40, ' started ', '-'x40 );

# ----------- subroutines --------------

sub ipGetIfaces()
{
	my %if = `/sbin/ifconfig` =~ /^(\S+).*?inet addr:(\S+)/gsm;
	%if;
}

sub ipSetAlias($)
{
	my $ip = pop;
	$log -> info( "ipSetAlias( $ip )" );

	sys_ex( "sudo ifconfig $IPGIVER_ALIAS $ip netmask 255.255.0.0 up 2>&1" ) or return 0;
	sys_ex( "sudo ip route add 10.0.0.0/8 via 10.254.0.254 src $ip 2>&1" ) or return 0;

	$log -> info( "   iflist: " . iflist() );
	$ip;
}

sub ipSetRandAlias()
{
	$log -> info( "ipSetRandAlias()" );

	socket( S, PF_INET, SOCK_STREAM, 0 )					or die "error: socket: $!\n";
	connect( S, sockaddr_in( $IPGIVER_PORT, inet_aton $IPGIVER_HOST ) )	or do
	{
		$log -> warning( "   connect to $IPGIVER_HOST:$IPGIVER_PORT failed: $!" );
		return 0;
	};

	my ( $rin, $rout );
	vec( $rin, fileno(S), 1 ) = 1;

	my ( $nf, $tl ) = select( $rout=$rin, undef, undef, $IPGIVER_TIMEOUT );
	if ( $nf == 0 )
	{
		$log -> warning( "   timeout ($IPGIVER_TIMEOUT sec)" );
		return 0;
	}

	defined recv( S, my $recv, 10240, 0 ) or do
	{
		$log -> warning( "   recv failed: $!" );
		return 0;
	};

	$recv =~ s/[\r\n]+//g;
	if ( $recv !~ /^10\.254\.\d+\.\d+/ )
	{
		$log -> warning( "   bad answer: '$recv'" );
		return 0;
	}

	close S;
	$log -> info( "   recieved '$&'" );

	return ipSetAlias( $& );
}

sub ipUnsetAlias()
{
	$log -> info( "ipUnsetAlias()" );
	
	my %if = ipGetIfaces();
	$log -> info( "   $IPGIVER_ALIAS already down" ) and return 1 unless $if{$IPGIVER_ALIAS};

	sys_ex( "sudo ifconfig $IPGIVER_ALIAS down 2>&1" ) or return 0;
	$log -> info( "   iflist: " . iflist() );
	1;
}

# --------------- internal routines -----------------

sub iflist()
{
	my %if = ipGetIfaces();
	join ', ', map { "$_: $if{$_}" } keys %if;
}

sub ipDebug()
{
	my %if = ipGetIfaces();
	print "$_ -> $if{$_}\n" for keys %if;
}

sub sys_ex($)
{
	my ( $c, $r ) = ( pop, 'OK' );

	my $s = `$c`;

	$s =~ s/[\r\n]+$//;
	$r = "FAIL: $s" if $? >> 8;

	$log -> info( sprintf "   exec: [ %-75s ] %s", $c, $r );

	$r eq 'OK';
}

# ipDebug();
# ipSetAlias( '10.254.76.67' );
# ipDebug();
# ipUnsetAlias();
# ipDebug();

1;

